% The PVTOL live!

clc,close all,clear all
message = ['The PVTOL live, may 1999, by Philippe Martin (martin@cas.ensmp.fr)'];
disp(message);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% TP ECP: initialiser ici les variables du controleur %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% Titre affiche sur la planche
mytitle = 'Mon controleur';

%%% Parametres du modele de commande
g = 10; 
m = ?; 
l  = ?; 
J = ?; 
alpha = ?;
a = ?;
b = ?;
epsi = ?; 

%%% Variables du generateur de trajectoire
%%% Utiliser ref0, ref1,... pour filtrer la souris (ref0 est la variable la plus integree 
%%% Utiliser (eventuellement) le tableau coeff et la fonction poly pour calculer 
%%% les coefficients K0, K1,... du filtre
ref0 = [0, 0]; ref1 = ref0; ref2 = ref1; ref3 = ref1;
K0 = ?;
K1 = ?;

%%% Gains du bouclage
%%% Utiliser (eventuellement) le tableau coeff  et la fonction poly pour calculer 
%%% les coefficients k0, k1,... et l0, l1, ...du bouclage
k0 = ?;
k1 = ?;

%%% Variables a initialiser en cas de bouclage dynamique
xi = ?;
xid = ?;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% TP ECP: fin de l'initialisation %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%  Initialize system variables
scale=.7; dt=1/15; flick=2; nbuf=500;  

x=0; z=10; theta=0; xd=0; zd=0; thetad=0; F1=49752; F2=F1;
currPt=[0,0]; mouse=0; go=1; i=0; crash=0;
buf=zeros(12,nbuf);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  Create figure, axes and graphical objects
fig = figure('BackingStore','off','Pointer','crosshair',...
   'NumberTitle','off','Name',['The PVTOL: ',mytitle],...
	'CloseRequestFcn','go=0; closereq',...
	'ResizeFcn',['pos=get(gcf,''Position'');'...
		'xlim=scale*[-2 2]/25*pos(3); ylim=scale*[0 2]/20*pos(4);'...
		'set(ax1,''XLim'',xlim,''YLim'',ylim);'...
		'set(box,''XData'',[xlim fliplr(xlim)],''YData'',[ylim(1) ylim ylim(2)])'],...
	'WindowButtonUpFcn',['mouse=0; set(href,''Color'',''b'');'...
		'set(fig,''WindowButtonMotionFcn'',''''); currPt=[0,0];']);

pos = get(gcf,'Position');
xlim=scale*[-2 2]/25*pos(3); 
ylim=scale*[0 2]/20*pos(4);

ax1 = axes('Box','on','Position',[.04 .4 .96 .6],'DataAspectRatio',[1 1 1],...
	'XLim',xlim,'YLim',ylim);
	
ax2 = axes('Box','on','Position',[.04 0.04 .32 .32],'DataAspectRatio',[1 1 1],...
	'XLim',10*[-1 1],'YLim',10*[-1 1],'XGrid','on','Ygrid','on');

ax3 = axes('Box','on','Position',[.4 0.04 .6 .32],'DataAspectRatio',[1 1 1],...
	'Xlim',100*[-1 1],'YLim',[0 90]);
	
box = patch('Parent',ax3,'XData',[xlim fliplr(xlim)],...
'YData',[ylim(1) ylim ylim(2)],'FaceColor','c');

vtol = [-5, 0, 5, .2, 0, -.2, -5; -.3, -.3,  -.3, .7, 2.2, .7, -.3];
vtol1 = [-5, 0, 5, .2, 0, -.2, -5]; 
vtol2 = [-.3, -.3,  -.3, .7, 2.2, .7, -.3];
hvtol = patch('Parent',ax1,'EraseMode','xor');

Ca = cos(atan(.1)); % tan(alpha)=.1
Sa = sin(atan(.1));
tmp = [-.5, .5, 0, -.5];
jet11 = 3+Ca*tmp;
jet12 = -.5-Sa*tmp;
hjet1 = patch('Parent',ax1,'EraseMode','xor','FaceColor','r');

jet21 = -3+Ca*tmp;
jet22 = -.5+Sa*tmp;
usejet = [jet11(3), jet12(3), jet21(3), jet22(3)];
hjet2 = patch('Parent',ax1,'EraseMode','xor','FaceColor','r');

hmark = line('Parent',ax3,'EraseMode','xor','LineWidth',5,'Color','r');
href = line('Parent',ax2,'EraseMode','xor','LineWidth',5,'Color','b',...
        'ButtonDownFcn',['mouse=1; set(href,''Color'',''r'');' ...
			'set(gcf,''WindowButtonMotionFcn'',''1;'')']);
		
%%% Create PVTOL Menu
hmenu = uimenu('Label','PVTOL');

uimenu('Parent',hmenu,'Label','Display','Accelerator','d',...
	'CallBack',['data=[buf(:,i+2:nbuf),buf(:,1:i+1)]; tt=linspace(-dt*nbuf,0,nbuf);'...
	'figure(2); zoom on; set(2,''Name'',''History'',''NumberTitle'',''off'');'...
	'subplot(511); plot(tt,data(1:2,:)); ylabel(''x, z'');'...
	'subplot(512); plot(tt,data(3:4,:),''-'',tt,data(11:12,:),''--''); ylabel(''xd, zd'');'...
	'subplot(513); plot(tt,data(5,:)*180/pi); ylabel(''theta'');'...
	'subplot(514); plot(tt,data(7:8,:)/100000); ylabel(''F1n, u1n'');'...
	'subplot(515); plot(tt,data(9:10,:)/100000); ylabel(''F2n, u2n'');'])

uimenu('Parent',hmenu,'Label','Reset','Accelerator','r',...
	'CallBack',['i=0; x=0; xd=0; z=10; zd=0; theta=0; thetad=0; xi=g; xid=0;'...
	'currPt=[0,0]; crash=0; ref0=[0, 0]; ref1=ref0; ref2=ref1; ref3=ref1;'...
	'F1=49752; F2=F1; set(ax1,''Color'',''w'');'])

uimenu('Parent',hmenu,'Label','Exit','Accelerator','e','Separator','on',...
	'CallBack','go=0; close all')
uimenu('Parent',hmenu,'Label','About PVTOL live...','Separator','on',...
	'CallBack','disp(message)')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Main loop
while go
if ~crash
	Cth = cos(theta);
	Sth = sin(theta);
	
	if mouse
		currPt=get(ax2,'CurrentPoint');
		currPt=currPt(1,1:2);
	end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% TP ECP: definir ici le controleur %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% N'UTILISER QUE LES VARIABLES AUTORISEES %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% Filtrage (eventuel) de la consigne fournie par la souris
% Le filtre utilise au plus les vecteurs ref0, ref1, ref2, ref3 et les
% gains K0, K1, K2, K3.
% ref0 est la variable la plus integree (donc en general la consigne de
% vitesse) et est affichee sur les courbes.
% Entree: vecteur currPt contenant la position courante de la souris
% Sortie: ref0

	ref0 = ref0 + dt*K0*(currPt-ref0);
		     
%%% Controleur
% Le controleur utilise au plus les parametres du modele de commande,
% les gains k0, k1, k2 et l0, l1, l2, et, s'il est dynamique, les etats xi et xid
% (et eventuellement les variables intermediaires xinew et xidnew).
% Entree: x, z, Cth, Sth, xd, zd, thetad 
% Sortie: vecteur des commandes u (en utilisant eventuellement le vecteur intermediaire v)

    u(1) = a*(g+ref0(2)-ref0(1)/10)/2;
    u(2) = a*(g+ref0(2)+ref0(1)/10)/2;   

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% TP ECP: fin du controleur %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
 	%%% Update state
	%%% lambda=15, J=45000, m=10000, l=4.5, tan(alpha)=.1, g=10
	tmp = dt*15;
	F1 = (F1+tmp*u(1))/(1+tmp);
	F2 = (F2+tmp*u(2))/(1+tmp);
	Fplus = (F1+F2)/9950.37;
	Fminus = (F1-F2)/9950.37;
	x = x+dt*xd;
	z = z+dt*zd;
	theta = theta+dt*thetad;
	xd = xd+dt*(.1*Fminus*Cth-Fplus*Sth-.05*(1+3*abs(Sth))*xd);
	zd = zd+dt*(.1*Fminus*Sth+Fplus*Cth-10-.05*(1+3*abs(Cth))*zd);
	thetad = thetad+dt*(Fminus-.05*thetad);
	
	%%% Store data
	buf(:,i+1)=[x; z; xd; zd; theta; thetad; F1; u(1); F2; u(2); ref0(1); ref0(2)];

	%%% Update graphics
	if ~mod(i,flick)
		if z<1
			if (zd<-1) | (abs(theta)>.2)
				set(ax1,'Color','r')
				crash = 1;
			end
			z = 1;
			zd = 0;
		end
		tmp = F1/9950.37;
		jet11(3) = usejet(1)-Sa*tmp;
		jet12(3) = usejet(2)-Ca*tmp;
		if F1<0
			Colorj1 = 'y'; 
		else
			Colorj1 = 'r';
		end
		tmp = F2/9950.37;
		jet21(3) = usejet(3) + Sa*tmp;
		jet22(3) = usejet(4) - Ca*tmp;
		if F2<0
			Colorj2 = 'y'; 
		else
			Colorj2 = 'r';
		end
		
		set(hjet1,'FaceColor',Colorj1,'XData',x+Cth*jet11-Sth*jet12,...
		'YData',z+Sth*jet11+Cth*jet12);
		set(hjet2,'FaceColor',Colorj2,'XData',x+Cth*jet21-Sth*jet22,...
		'YData',z+Sth*jet21+Cth*jet22);
		set(hvtol,'XData',x+Cth*vtol1-Sth*vtol2,'YData',z+Sth*vtol1+Cth*vtol2)	
		set(href,'XData',[ref0(1)-.5,ref0(1)+.5],'YData',[ref0(2),ref0(2)])
		set(hmark,'XData',[x-1,x+1],'YData',[z,z])
		drawnow
	end
	i=mod(i+1,nbuf);
else
	pause(1)
end	
end



